import { defineComponent, ref, reactive, watch, toRef, onMounted, onUnmounted, h, Teleport } from 'vue';

function toVueComponent(HFC, lowercase) {
    HFC.propTypes = HFC.propTypes || {};
    const propKeys = Object.keys(HFC.propTypes.attrs || {});
    const eventKeys = Object.keys(HFC.propTypes.events || {});
    const slotKeys = Object.keys(HFC.propTypes.slots || {});
    const propKeyMap = {};
    const eventKeyMap = {};
    const slotKeyMap = {};
    propKeys.forEach((k) => {
        propKeyMap[lowercase ? k.toLowerCase() : k] = k;
    });
    eventKeys.forEach((k) => {
        eventKeyMap[lowercase
            ? "on" + k[0].toUpperCase() + k.slice(1).toLowerCase()
            : "on" + k[0].toUpperCase() + k.slice(1)] = k;
    });
    slotKeys.forEach((k) => {
        slotKeyMap[lowercase ? k.toLowerCase() : k] = k;
    });
    const observePropKeys = Object.keys(propKeyMap);
    const observeEventKeys = Object.keys(eventKeyMap);
    const observeSlotKeys = Object.keys(slotKeyMap);
    return defineComponent({
        props: observePropKeys,
        setup(vueProps, { slots: vueSlots, attrs: vueAttrs }) {
            const container = ref();
            const events = {};
            observeEventKeys.forEach((key) => {
                const event = vueAttrs[key];
                if (!event)
                    return;
                events[eventKeyMap[key]] = event;
            });
            const slots = {};
            const renderedSlots = reactive({});
            observeSlotKeys.forEach((key) => {
                const slot = vueSlots[key];
                if (!slot)
                    return;
                slots[slotKeyMap[key]] = function (container, args) {
                    if (!container) {
                        delete renderedSlots[key];
                        return;
                    }
                    const slotItem = renderedSlots[key];
                    if (!slotItem) {
                        renderedSlots[key] = { name: key, container, slot, args };
                        return;
                    }
                    if (slotItem.container !== container) {
                        slotItem.container = container;
                    }
                    slotItem.args = args;
                };
            });
            let hfc;
            const attrs = {};
            observePropKeys.forEach((key) => {
                if (!vueProps[key])
                    return;
                attrs[propKeyMap[key]] = vueProps[key];
                watch(toRef(vueProps, key), (curr, prev) => {
                    var _a;
                    attrs[propKeyMap[key]] = curr;
                    (_a = hfc.changed) === null || _a === void 0 ? void 0 : _a.call(hfc, "attr", propKeyMap[key], prev, curr);
                }, { deep: true });
            });
            hfc = new HFC({
                attrs,
                events,
                slots,
            });
            onMounted(() => {
                hfc.connected(container.value);
            });
            onUnmounted(() => {
                var _a;
                (_a = hfc.disconnected) === null || _a === void 0 ? void 0 : _a.call(hfc);
            });
            return () => [
                h("div", { ref: container }),
                Object.keys(renderedSlots).map((key) => {
                    const item = renderedSlots[key];
                    return h(Teleport, { to: item.container }, item.slot(item.args));
                }),
            ];
        },
    });
}

export { toVueComponent as default };
